YUI(Yconfig).use(
  'node', 'tabview', 'datatable', 'history', 'recordset-base', 'recordset-filter', 'io', 'json', 'linaro-trivia',
  function (Y) {
    var history = new Y.HistoryHash(),
    tabview = new Y.TabView({srcNode: '#build-tabs'});

    tabview.render();

    tabview.after(
      'selectionChange',
      function (e) {
        var val = e.newVal.get('srcNode').get('id').replace(/-tab/, '');
        history.addValue('builds', val);
      }
    );

    function selectTabByName (name) {
      var node = Y.one('#' + name + '-tab');
      if (!node) return;
      tabview.selectChild(Y.Widget.getByNode(node).get('index'));
    }

    function logInAndGoToMyBuilds (self) {
      window.location.href = globalConfig.loginURL + '?next=' + location.pathname + '%23builds=mine';
    }

    Y.on(
      'history:change',
      function (e) {
        if (e.src === Y.HistoryHash.SRC_HASH) {
          if (e.changed.builds) {
            if (e.changed.builds.newVal == 'mine' && !globalConfig.userName) {
              // you can get here by, for example, switching away from the my tab in window 1,
              // logging out in window 2, then using the back button in window 1
               logInAndGoToMyBuilds();
            } else {
              selectTabByName(e.changed.builds.newVal);
            }
          } else if (e.removed.builds) {
            tabview.selectChild(0);
          }
        }
      });

    tabview.on(
      'render',
      function (e) {
        if (!globalConfig.userName) {
          var myBuildsTab = tabview.item(1);
          myBuildsTab.on('click', function (e) { e.preventDefault(); });
          var link = myBuildsTab.get('boundingBox').one('a');
          link.setStyle('color', 'grey');
          link.setAttribute('href', globalConfig.loginURL + '?next=' + location.pathname + '%23builds=mine');
        }
      });

    if (history.get('builds') == 'mine' && !globalConfig.userName) {
      logInAndGoToMyBuilds();
    } else {
      selectTabByName(history.get('builds') || 'official');
    }

    var treeExp = 'jobs[name,lastBuild[result,timestamp,duration]]';

    function cmp(a, b) {
      if (a > b) {
        return 1;
      } else if (a < b) {
        return -1;
      } else {
        return 0;
      }
    }

    function compareBuilds(recA, recB) {
      // this is probably not really right, but it's a start.
      var startA = recA.start;
      var startB = recB.start;
      if (startA == '') {
        if (startB == '') {
          return cmp(recA.name, recB.name);
        } else {
          return -1;
        }
      } else {
        if (startB == '') {
          return 1;
        } else {
          return -cmp(startA, startB); // textual comparison!  do better here
        }
      }
    }

    function MyDataTableDataFilter() {
      MyDataTableDataFilter.superclass.constructor.apply(this, arguments);
    }

    Y.mix(
      MyDataTableDataFilter, {
        NS: "datafilter",
        NAME: "myDataTableDataFilter",

        ATTRS: {
          keys: {},
          filter: {}
        }
      });

    Y.extend(
      MyDataTableDataFilter, Y.Plugin.Base, {
        newData: function (records) {
          var recordset = new Y.Recordset({records: records});
          recordset.plug(Y.Plugin.RecordsetFilter);
          var filter = this.get('filter');
          if (filter) {
            recordset = recordset.filter.filter(filter);
            recordset.plug(Y.Plugin.RecordsetFilter);
          }
          this.get("host").set("recordset", recordset);
          this.unfiltered_data = recordset;
          this.status = "Any";
          this.search = '';
        },
        _filter: function () {
          // Could optimize this by filtering the existing record set if the new search
          // parameters are a restriction
          var recordset = this.unfiltered_data.filter.filter(
            Y.bind(function (item) {
                     if (this.status != 'Any') {
                       if (item.getValue('status') != this.status) return false;
                     }
                     return item.getValue('name').search(this.search) >= 0;
                   }, this)
          );
          this.get("host").set("recordset", recordset);
        },
        filterStatus: function (status) {
          this.status = status;
          this._filter();
        },
        filterName: function (search) {
          this.search = search;
          this._filter();
        }
      });

    function formatCell (o) {
      return '<a href="builds/' + o.data.name + '/">' + o.value + '</a>';
    }

    function makeDT(id, filter) {

      var columns = [
        {label:"Status", key: 'status'},
        {label:"Name", key: 'name'},
        {label:"Build Started", key: 'start'},
        {label:"Build Finished", key: 'finish'}
      ];

      Y.Array.each(columns, function (o) { o.formatter = formatCell; });

      var dt = new Y.DataTable.Base(
        {
          columnset: columns
        });

      dt.render(id);

      dt.after(
        'recordsetChange',
        function (e) {
          var rows = e.target.get('contentBox').all('tbody tr');

          // there must be better ways to do this...
          rows.on(
            'click',
            function (e) {
              var buildName = e.target.ancestor('tr').all('td').item(1).get('text');
              location.href = 'builds/'+buildName + '/';
            });
        }
      );

      var tableNode = Y.one(id);

      var selectNode = tableNode.ancestor('div').one('select');

      if (selectNode) {
        selectNode.on(
          'change', function (e) {
            dt.datafilter.filterStatus(e.target.get('value'));
          }
        );
      }

      tableNode.ancestor('div').one('.search').on(
        'keyup', function (e) {
          dt.datafilter.filterName(e.target.get('value'));
        }
      );

      dt.plug(MyDataTableDataFilter, {
                filter: filter
              });
      return dt;
    }

    var userRegexp = (
      globalConfig.userName ? new RegExp('^~' + globalConfig.userName + '/') : null);

    var tables = [
      makeDT(
        "#official-build-table",
        function (item) { return item.getValue('name').search(/^~linaro-android\//) == 0; }
      ),
      makeDT(
        "#my-build-table",
        (userRegexp ?
         function (item) { return item.getValue('name').search(userRegexp) == 0; } :
         function (item) { return false; })
      ),
      makeDT("#all-build-table")
    ];

    function extractJobsFromResponse(data) {
      var response = Y.JSON.parse(data);
      var results = [];
      for (var i = 0; i < response.jobs.length; i++) {
        var job = response.jobs[i];
        var under_index = job.name.search('_');
        if (under_index < 0) continue;
        var status;
        if (!job.lastBuild) {
          status = '';
        } else if (job.lastBuild.result === null) {
          status = 'RUNNING';
        } else if (job.lastBuild.result === "SUCCESS") {
          status = 'OK';
        } else if (job.lastBuild.result === "FAILURE") {
          status = 'FAILED';
        } else if (job.lastBuild.result === "ABORTED") {
          status = 'ABORTED';
        } else {
          status = '???' + job.lastBuild.result;
        }
        job = {
          name:  '~' + job.name.substring(0, under_index) + '/' + job.name.substring(under_index+1),
          start: job.lastBuild ? Y.linaro.formatDateFromTimestamp(job.lastBuild.timestamp) : '',
          finish: (job.lastBuild && job.lastBuild.duration)
            ? Y.linaro.formatDateFromTimestamp(job.lastBuild.timestamp + job.lastBuild.duration) : '',
          status: status
        };
        results.push(job);
        results.sort(compareBuilds);
      }
      return results;
    }

    Y.io(
      globalConfig.jenkinsURL + "/api/json?tree=" + treeExp,
      {
        method: "POST",
        on: {
          success: function (e, response) {
            var jobs = extractJobsFromResponse(response.responseText);
            for (var i = 0; i < tables.length; i++) {
              tables[i].datafilter.newData(jobs);
            }
          }
        }
      }
    );
  }
);
